package sun.net.www.http;

import java.util.Random;
import sun.util.logging.PlatformLogger;
import java.io.FileWriter;
import java.net.URL;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.BufferedReader;
import java.io.FileReader;
import java.security.AccessController;
import sun.net.NetProperties;
import java.security.PrivilegedAction;
import java.util.regex.Pattern;
import java.util.ArrayList;
import java.io.BufferedWriter;
import java.io.File;

public class HttpCapture
{
    private File file;
    private boolean incoming;
    private BufferedWriter out;
    private static boolean initialized;
    private static volatile ArrayList<Pattern> patterns;
    private static volatile ArrayList<String> capFiles;
    
    private static synchronized void init() {
        HttpCapture.initialized = true;
        String fileName = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return NetProperties.get("sun.net.http.captureRules");
            }
        });
        if (fileName != null && !fileName.isEmpty()) {
            BufferedReader bufferedReader;
            try {
                bufferedReader = new BufferedReader(new FileReader(fileName));
            }
            catch (FileNotFoundException ex) {
                return;
            }
            try {
                for (String s = bufferedReader.readLine(); s != null; s = bufferedReader.readLine()) {
                    String trim = s.trim();
                    if (!trim.startsWith("#")) {
                        String[] split = trim.split(",");
                        if (split.length == 2) {
                            if (HttpCapture.patterns == null) {
                                HttpCapture.patterns = new ArrayList();
                                HttpCapture.capFiles = new ArrayList();
                            }
                            HttpCapture.patterns.add(Pattern.compile(split[0].trim()));
                            HttpCapture.capFiles.add(split[1].trim());
                        }
                    }
                }
            }
            catch (IOException ex2) {}
            finally {
                try {
                    bufferedReader.close();
                }
                catch (IOException ex3) {}
            }
        }
    }
    
    private static synchronized boolean isInitialized() {
        return HttpCapture.initialized;
    }
    
    private HttpCapture(File file, URL obj) {
        this.file = null;
        this.incoming = true;
        this.out = null;
        this.file = file;
        try {
            (this.out = new BufferedWriter(new FileWriter(this.file, true))).write("URL: " + obj + "\n");
        }
        catch (IOException t) {
            PlatformLogger.getLogger(HttpCapture.class.getName()).severe(null, t);
        }
    }
    
    public synchronized void sent(int c) throws IOException {
        if (this.incoming) {
            this.out.write("\n------>\n");
            this.incoming = false;
            this.out.flush();
        }
        this.out.write(c);
    }
    
    public synchronized void received(int c) throws IOException {
        if (!this.incoming) {
            this.out.write("\n<------\n");
            this.incoming = true;
            this.out.flush();
        }
        this.out.write(c);
    }
    
    public synchronized void flush() throws IOException {
        this.out.flush();
    }
    
    public static HttpCapture getCapture(URL url) {
        if (!isInitialized()) {
            init();
        }
        if (HttpCapture.patterns == null || HttpCapture.patterns.isEmpty()) {
            return null;
        }
        String string = url.toString();
        int i = 0;
        while (i < HttpCapture.patterns.size()) {
            if (((Pattern)HttpCapture.patterns.get(i)).matcher(string).find()) {
                String pathname = (String)HttpCapture.capFiles.get(i);
                Object o;
                if (pathname.indexOf("%d") >= 0) {
                    Random random = new Random();
                    do {
                        o = new File(pathname.replace("%d", Integer.toString(random.nextInt())));
                    } while (o.exists());
                }
                else {
                    o = new File(pathname);
                }
                return new HttpCapture(o, url);
            }
            else {
                ++i;
            }
        }
        return null;
    }
    
    static {
        HttpCapture.initialized = false;
        HttpCapture.patterns = null;
        HttpCapture.capFiles = null;
    }
}
